#include <iostream>
#include <cstdio>
#include <cstddef>
#include <climits>
#include <vector>
#include <iostream>
#include <mpi.h>
#include "datastructures/Body.hpp"

using namespace std;
using namespace nbody;

int main(int argc, char* argv[]) {
    const int tag = 13;
    int size, rank;

    MPI_Init(&argc, &argv);
    MPI_Comm_size(MPI_COMM_WORLD, &size);

    if (size < 2) {
        fprintf(stderr,"Requires at least two processes.\n");
    }

	int bodyBlocklengths[6] = {1, 3, 3, 3, 1, 1};
	MPI_Datatype bodyDatatypes[6] = {MPI_UNSIGNED_LONG, MPI_DOUBLE, MPI_DOUBLE, MPI_DOUBLE, MPI_DOUBLE, MPI_INT};
	MPI_Aint bodyOffsets[6];
	MPI_Datatype bodyType;
    bodyOffsets[0] = offsetof(Body, id);
    bodyOffsets[1] = offsetof(Body, position);
    bodyOffsets[2] = offsetof(Body, velocity);
    bodyOffsets[3] = offsetof(Body, acceleration);
    bodyOffsets[4] = offsetof(Body, mass);
    bodyOffsets[5] = offsetof(Body, refinement);
    MPI_Type_create_struct(6, bodyBlocklengths, bodyOffsets, bodyDatatypes, &bodyType);
    MPI_Type_commit(&bodyType);

    /*
	int boxBlocklengths[2] = {3, 3};
	MPI_Datatype boxDatatypes[2] = {MPI_DOUBLE, MPI_DOUBLE};
	MPI_Aint boxOffsets[2];
	MPI_Datatype boxType;
    bodyOffsets[0] = offsetof(Body, id);
    bodyOffsets[1] = offsetof(Body, position);
    MPI_Type_create_struct(2, boxBlocklengths, boxOffsets, boxDatatypes, &boxType);
    MPI_Type_commit(&boxType);
    */
/*
    const int bitems = 6;
    int          bblocklengths[6] = {1, 3, 3, 3, 1, 1};
    MPI_Datatype btypes[6] = {MPI_UNSIGNED_LONG, MPI_DOUBLE, MPI_DOUBLE, MPI_DOUBLE, MPI_DOUBLE, MPI_INT};
    MPI_Datatype mpi_body_type;
    MPI_Aint     boffsets[6];

    boffsets[0] = offsetof(Body, id);
    boffsets[1] = offsetof(Body, position);
    boffsets[2] = offsetof(Body, velocity);
    boffsets[3] = offsetof(Body, acceleration);
    boffsets[4] = offsetof(Body, mass);
    boffsets[5] = offsetof(Body, refinement);

    MPI_Type_create_struct(bitems, bblocklengths, boffsets, btypes, &mpi_body_type);
    MPI_Type_commit(&mpi_body_type);
    */



    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
    if (rank == 0) {
    	vector<Body> send = dubinskiParse(argv[1]);


        for (int i = 0; i < send.size(); i++) {
        	printBody(0, send[i]);
        }


        const int dest = 1;
        int bodySize = send.size();
        MPI_Send(&bodySize,   1, MPI_INT, dest, tag, MPI_COMM_WORLD);
        MPI_Send(&send[0],   bodySize, bodyType, dest, tag, MPI_COMM_WORLD);

    	/*
        vector<Car> send;

        send.push_back(Car());
        send.push_back(Car());
        send[0].shifts = 4;
        send[0].topSpeed = 100;
        send[0].pos[0] = 0.5;
        send[0].pos[1] = 0.6;
        send[0].pos[2] = 0.7;

        send[1].shifts = 5;
        send[1].topSpeed = 200;
        send[1].pos[0] = 1.5;
        send[1].pos[1] = 1.6;
        send[1].pos[2] = 1.7;

        const int dest = 1;
        MPI_Send(&send[0],   2, mpi_car_type, dest, tag, MPI_COMM_WORLD);
        printf("Rank %d: sent structure car\n", rank);
        */
    }
    if (rank == 1) {
        MPI_Status status;
        const int src=0;

        vector<Body> recv;
        int bodySize;

        MPI_Recv(&bodySize,   1, MPI_INT, src, tag, MPI_COMM_WORLD, &status);
        recv.reserve(bodySize);
        MPI_Recv(&recv[0],   bodySize, bodyType, src, tag, MPI_COMM_WORLD, &status);

        for (int i = 0; i < bodySize; i++) {
        	printBody(1, recv[i]);
        }

    	/*
        MPI_Status status;
        const int src=0;

        vector<Car> recv;

        recv.reserve(2);
        MPI_Recv(&recv[0],   2, mpi_car_type, src, tag, MPI_COMM_WORLD, &status);
        printf("Rank %d: Received: shifts = %d topSpeed = %d | %f | %f | %f\n", rank,
                 recv[0].shifts, recv[0].topSpeed, recv[0].pos[0], recv[0].pos[1], recv[0].pos[2]);
        printf("Rank %d: Received: shifts = %d topSpeed = %d | %f | %f | %f\n", rank,
                 recv[1].shifts, recv[1].topSpeed, recv[1].pos[0], recv[1].pos[1], recv[1].pos[2]);
                 */

    }
    MPI_Type_free(&bodyType);
    MPI_Finalize();
	return 0;
}
