#include <PthreadSimulation.hpp>
#include <iostream>
#include <unistd.h>
#include "BarnesHutTreeThreaded.hpp"

namespace nbody {
	using namespace std;

	PthreadSimulation::PthreadSimulation() {
	}

	PthreadSimulation::~PthreadSimulation() {
		delete this->tree;
	}

	void PthreadSimulation::initialize(string inputFile) {
		if (!this->readInputData(inputFile)) {
			cerr << "input file error" << endl;
			exit(-1);
		}
		int numThreads = sysconf(_SC_NPROCESSORS_ONLN);
		for (int i = 0; i < numThreads; i++) {
			pthread_t thread;

			this->threads.push_back(thread);
		}
		this->tree = new BarnesHutTreeThreaded(0);
		this->tree->simulation = this;
		this->tree->build(this->bodies);
		//cout << "NON: " << this->tree->numberOfNodes() << endl;
		//this->tree->print(0);
	}

	void PthreadSimulation::cleanup() {
		delete this->tree;
		this->tree = NULL;
		pthread_exit(NULL);
	}

	int PthreadSimulation::getNumberOfProcesses() {
		return this->threads.size();
	}

	int PthreadSimulation::getProcessId() {
		if (this->threads.empty()) {
			return 0;
		}
		vector<pthread_t>::iterator it = this->threads.begin();
		int id = 0;

		while (*it != pthread_self() && it != this->threads.end()) {
			id++;
			it++;
		}
		if (it != this->threads.end()) {
			return id;
		} else {
			return -1;
		}
	}

	void PthreadSimulation::runStep() {
		//Box domain = this->tree.getRootBB();

		printBB(0, this->tree->getRootBB());
		((BarnesHutTreeThreaded*) this->tree)->computeMove();
		this->tree->rebuild();
		cout << "-----" << endl;
		printBB(0, this->tree->getRootBB());
		/*
		this->tree->rebuild();
		this->tree->computeForces();
		this->tree->advance();
		*/
	}

}
